@@ -12,15 +12,13 @@
 # **  For license terms, refer to distribution info            **
 # \***************************************************************/
 
-import time
-import random
-import requests
 import json
-import string
-import yaml
-import os
-from jinja2 import Template
+import re
 import collections
+import os
+import yaml
+from jinja2 import Template
+import requests
 from cvapianalyser import CommunityEdition
 from openapispecdiff import OpenApiSpecDiff
 
@@ -47,45 +45,38 @@ def key_lookup(key, var):
             if k:
                 paths.append(".".join(k))
     return paths
-    # paths.append(".".join(k))
-    #
-    # print("\n\n\n")
-    # if hasattr(var, 'items'):
-    #     for k, v in var.items():
-    #
-    #         if k == key:
-    #             yield path.replace(k + ".", "")
-    #
-    #         path += k + "."
-    #         if isinstance(v, dict):
-    #             for result in key_lookup(key, v, path):
-    #                 yield result
-    #         elif isinstance(v, list):
-    #             for d in v:
-    #                 for result in key_lookup(key, d, path):
-    #                     yield result
 
 
 class CloudvectorDAST(object):
+    """Library for creating pytest functional and fuzz test cases for a given Open API spec.
+
+    Input:
+
+    """
     def __init__(self, APISpecOne, APISpecTwo, ce_host, ce_username, ce_password, config_file, cover_only_diff="n",
                  input_params_file=None, do_fuzz=False):
 
         custom_validations = None
         self.static_headers_from_cfg = {}
+        self.cv_config = None
+        self.cvdast_to_auth_info = {}
+        self.params_from_spec = {}
         if config_file:
             if os.path.exists(config_file):
-                cv_config = self._parse_cv_config(config_file)
-                ce_host = cv_config["ce_setup"]["ce_host"]
-                ce_username = cv_config["ce_setup"]["ce_username"]
-                custom_validations = cv_config.get("custom_validations")
-                self.static_headers_from_cfg = cv_config.get("static_headers")
+                self.cv_config = self._parse_cv_config(config_file)
+                ce_host = self.cv_config["ce_setup"]["ce_host"]
+                ce_username = self.cv_config["ce_setup"]["ce_username"]
+                custom_validations = self.cv_config.get("custom_validations")
+                self.static_headers_from_cfg = self.cv_config.get("static_headers")
+                if self.cv_config.get("authentication"):
+                  self.cvdast_to_auth_info = self.cv_config.get("authentication")
         if APISpecOne:
             self.apispec_one_path = APISpecOne
         else:
             self.apispec_one_path = None
         self.apispec_two_path = APISpecTwo
         self.openapispec_obj = OpenApiSpecDiff.OpenApiSpecDiff(self.apispec_one_path, self.apispec_two_path)
-        self.ceobj = CommunityEdition.CommunityEdition("http://" + ce_host, ce_username, ce_password)
+        self.ceobj = CommunityEdition.CommunityEdition(ce_host, ce_username, ce_password)
         print("\n\t\t\t\t\t\t\t\t\t----------------- DAST - For CloudVector APIShark events "
               "-----------------")
         # self.regenerate_traffic(self._get_changed_apis())
@@ -130,6 +121,8 @@ class CloudvectorDAST(object):
             self.create_fuzz_test_methods(cv_events)
             # self.create_fuzzfixtures()
         self.create_assertions(fuzzing=do_fuzz)
+        # if self.authentication_api_from_spec:
+        #     self._update_params_captured_for_auth()
 
     def _load_input_from_files(self, input_file):
         input_vars = {}
@@ -147,28 +140,47 @@ class CloudvectorDAST(object):
                 input_vars[str(key).strip()] = value
         return input_vars
 
+    # def _update_params_captured_for_auth(self, auth_info):
+    #     with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "tests/authcreds.json"),"w+") as fobj:
+    #         json.dumps(auth_info)
+
     def _prepare_spec_for_test_generation(self, input_spec):
         parsed_new_spec = self._get_spec_parsed(input_spec)
         apis_from_spec = []
         if "servers" in parsed_new_spec:
+            from urllib.parse import urlparse
             baseurl = parsed_new_spec["servers"][0]["url"]
-            host = ""
+            host = urlparse(baseurl).netloc
+            if host:
+                baseurl = baseurl.split(host)[1:]
+
         else:
-            baseurl = parsed_new_spec.get("host","") + parsed_new_spec.get("basePath","")
+            baseurl = parsed_new_spec.get("host", "") + parsed_new_spec.get("basePath", "")
             host = parsed_new_spec.get("host")
+        if type(baseurl) is list:
+            baseurl = baseurl[0]
         for api, info in parsed_new_spec["paths"].items():
-            x = {}
             for method, minfo in info.items():
-                x["method"] = method
+                x = {}
+                x["method"] = method.lower()
                 x["host"] = host
                 x["header"] = {}
                 x["params"] = []
                 x["body"] = {}
                 x["rsp_body"] = {}
-                x["http-req-url"] = api
-                x["url"] = baseurl + str(api)
+                x["url"] = ""
+                x["http-req-url"] = baseurl+api
+                if baseurl:
+                    x["url"] = baseurl+api
+                if host:
+                    x["url"] = host + baseurl + api
+                for _ in x["url"].split("/"):
+                    if "{" in _ and "}" in _:
+                        _ = str(_).replace("{", "").replace("}", "")
+                        x["params"].append(_)
+                #x["params"] += url_params
                 x["url"] = x["url"].replace("{", "{{").replace("}", "}}")
-                for _ in minfo["parameters"]:
+                for _ in minfo.get("parameters", []):
                     if _["in"] == "header":
                         x["header"][_["name"]] = ""
                     elif _["in"] == "query":
@@ -176,10 +188,14 @@ class CloudvectorDAST(object):
                             x["url"] += "?" + _["name"] + "={{" + _["name"] + "}}"
                         else:
                             x["url"] += "&" + _["name"] + "={{" + _["name"] + "}}"
+                        x["params"].append(_["name"])
                     else:
                         x["params"].append(_["name"])
                         x["body"][_["name"]] = ""
-            apis_from_spec.append([x, None])
+                self.params_from_spec[api] = {_:[] for _ in x["params"]}
+                # if str(self.cv_config["authentication"]["api"]).lower() in str(x["url"]).lower():
+                #     self.cvdast_to_auth_info = x
+                apis_from_spec.append([x, None])
         return apis_from_spec
 
     def _parse_cv_config(self, path):
@@ -189,16 +205,31 @@ class CloudvectorDAST(object):
 
     def _scan_input_spec(self, input_path):
         input_spec = self._get_spec_parsed(input_path)
+        if "servers" in input_spec:
+            from urllib.parse import urlparse
+            baseurl = input_spec["servers"][0]["url"]
+            host = urlparse(baseurl).netloc
+            if host:
+                baseurl = baseurl.split(host)[1:]
+
+        else:
+            baseurl = input_spec.get("host", "") + input_spec.get("basePath", "")
+            host = input_spec.get("host")
+        if type(baseurl) is list:
+            baseurl = baseurl[0]
         params_info = {}
         for api, info in input_spec["paths"].items():
+            api = baseurl+api
             if api not in params_info:
                 params_info[api] = {}
-            params = []
             for method, paraminfo in info.items():
+                params = []
+                if method not in params_info[api]:
+                    params_info[api][method] = []
                 for each in paraminfo.get("parameters", []):
                     if each.get("in") != "header":
                         params.append(each.get("name"))
-            params_info[api] = params
+                params_info[api][method] = params
         return params_info
 
     def _get_spec_parsed(self, input_path):
@@ -269,8 +300,38 @@ class CloudvectorDAST(object):
         print(self.validate_pycode_for_syntax(code))
         if not os.path.exists("tests"):
             os.mkdir("tests")
+        auth_api = None
+        auth_api_headers = None
+        auth_api_payload = None
+        auth_api_response_key = None
+        auth_inputs = ["host"]
+        if self.cvdast_to_auth_info:
+            auth_api = self.cvdast_to_auth_info["api_url"]
+            auth_api_headers = self.cvdast_to_auth_info["headers"]
+            auth_api_payload = self.cvdast_to_auth_info["payload"]
+            for k, v in auth_api_headers.items():
+                template_found = re.search(r"\[\'([A-Za-z0-9_]+)\'\]", str(v))
+                if template_found:
+                    auth_inputs.append(template_found.group(1))
+                    auth_api_headers[k] = auth_inputs[-1]
+            for k, v in auth_api_payload.items():
+                print(k)
+                print(v)
+                template_found = re.search(r"\[\'([A-Za-z0-9_]+)\'\]", str(v))
+                if template_found:
+                    auth_inputs.append(template_found.group(1))
+                    auth_api_payload[k] = auth_inputs[-1]
+            auth_api_response_key = self.cvdast_to_auth_info["response_key"]
+        for api,params_info in self.params_from_spec.items():
+            if api not in self.params_captured_in_traffic:
+                self.params_captured_in_traffic[api] = {}
+            for k, v in params_info.items():
+                if k not in self.params_captured_in_traffic[api]:
+                    self.params_captured_in_traffic[api][k]=v
         with open("tests/conftest.py", 'w+') as fh:
-            fh.write(template.render(api_info=self.params_captured_in_traffic))
+            fh.write(template.render(api_info=self.params_captured_in_traffic,AUTH_API=auth_api,
+                                     AUTH_API_HEADERS=auth_api_headers,AUTH_API_PAYLOAD=json.dumps(auth_api_payload),
+                                     AUTH_RESP_KEY=auth_api_response_key, AUTH_INPUTS=auth_inputs))
         print("\n\t\t......done creating pytest fixtures (conftest.py)")
 
     def create_fuzzfixtures(self):
@@ -304,7 +365,7 @@ class CloudvectorDAST(object):
     def _create_assertions_map(self, url, params, req_payload, resp_payload):
         assertions_map = {}
         if url not in self.assertions_map:
-            url = url.replace("/", "_").replace("-", "_").replace("{","").replace("}","").split("?")[0]
+            url = url.replace("/", "_").replace("-", "_").replace("{", "").replace("}", "").split("?")[0]
             self.assertions_map[url] = {}
 
         for param in params:
@@ -343,47 +404,56 @@ class CloudvectorDAST(object):
         print("\n\ncreating Pytest test methods....\n")
         new_spec_info = self._scan_input_spec(self.apispec_two_path)
         hosts = new_spec_info.get("servers", {}).get("url", [])
+        #print(cv_events)
         for _ in cv_events:
             event = _[0]
             if _[1]:
                 if _[1]["attributes"]["http_rsp_status_code"] not in ["200", "201"]:
                     continue
-            if event["url"] not in apis_to_be_tested:
-                api = str(event["http-req-url"]).lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
+            if event["url"]:
+                api = str(event["http-req-url"]).lower().lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
                     0]
-                apis_to_be_tested[api] = {}
-                apis_to_be_tested[api]["method"] = event["method"]
-                for k, v in event["header"].items():
-                    if k in self.static_headers_from_cfg:
-                        event["header"][k] = self.static_headers_from_cfg[k]
-                apis_to_be_tested[api]["header"] = event["header"]
-                apis_to_be_tested[api]["url"] = event["url"] #str(event["host"]).lower() + event["url"]
-                apis_to_be_tested[api]["params"] = new_spec_info.get(str(event["http-req-url"]).replace("//", "/"))
-                if apis_to_be_tested[api]["params"] is None:
-                    apis_to_be_tested[api]["params"] = []
-                apis_to_be_tested[api]["params"] = list(set(apis_to_be_tested[api]["params"]))
+                if api not in apis_to_be_tested:
+                    apis_to_be_tested[api] = {}
+                #apis_to_be_tested[api]["method"] = event["method"]
+                method = event["method"].lower()
+                if method not in apis_to_be_tested[api]:
+                    apis_to_be_tested[api][method] = {}
+                if self.static_headers_from_cfg:
+                    for k, v in event["header"].items():
+                        if k in self.static_headers_from_cfg:
+                            event["header"][k] = str(self.static_headers_from_cfg[k])
+                if "header" not in apis_to_be_tested[api][method]:
+                    apis_to_be_tested[api][method]["header"] = {}
+                apis_to_be_tested[api][method]["header"].update(event["header"])
+                apis_to_be_tested[api][method]["url"] = event["url"]  # str(event["host"]).lower() + event["url"]
+                if apis_to_be_tested[api][method].get("params"):
+                    known_params = apis_to_be_tested[api][method].get("params")
+                else:
+                    known_params = []
+                apis_to_be_tested[api][method]["params"] = new_spec_info.get(str(event["http-req-url"]).replace("//", "/"),{}).get(method,{})
+                if apis_to_be_tested[api][method]["params"] is None:
+                    apis_to_be_tested[api][method]["params"] = []
+                apis_to_be_tested[api][method]["params"] = list(set(apis_to_be_tested[api][method]["params"]))
+                apis_to_be_tested[api][method]["params"].append("host")
+                apis_to_be_tested[api][method]["params"].append("url_prefix")
+                apis_to_be_tested[api][method]["params"].append("access_token")
                 host_url = str(event["host"]).lower()
                 hosts.append(host_url)
-                self._create_assertions_map(event["http-req-url"], apis_to_be_tested[api]["params"], event["body"],
+                self._create_assertions_map(event["http-req-url"], apis_to_be_tested[api][method]["params"], event["body"],
                                             event["rsp_body"])
         with open(os.path.join(templates_dir, 'test_api.j2')) as file_:
             template = Template(file_.read())
-
-        # if len(hosts) > 1:
-        #     host_url = self._get_hosts_input(hosts)
-        # else:
-        #     host_url = hosts[-1]
-        host_url = list(set([_ for _ in hosts if _ != '']))
+        #host_url = [list(set([_ for _ in hosts if _ != '']))[-1]]
         for k, v in apis_to_be_tested.items():
             # code = template.render(api_info=apis_to_be_tested[k], api_name=k, host_url=host_url)
             filename = str(k).replace("/", "_").replace("{", "").replace("}", "")
             if not os.path.exists("tests"):
                 os.mkdir("tests")
             extra_validations = self._create_custom_validations(params_for_custom_validations,
-                                                                apis_to_be_tested[k]["params"])
-            # print("????>>>>>"+str(extra_validations))
+                                                                apis_to_be_tested[k])
             with open("tests/test_" + str(filename) + ".py", 'w+') as fh:
-                fh.write(template.render(api_info=apis_to_be_tested[k], api_name=k, host_url=host_url,
+                fh.write(template.render(apis_metadata=apis_to_be_tested[k], api_name=k, host_url=host_url,
                                          custom_validations=extra_validations))
             files_created.add("test_" + str(filename))
         print("\n\t\t......done creating pytest methods: " + str(files_created))
@@ -426,12 +496,7 @@ class CloudvectorDAST(object):
             fuzzing_details = self._get_fuzz_details(type)
             with open(os.path.join(templates_dir, 'fuzz_test.j2')) as file_:
                 template = Template(file_.read())
-            # print(apis_to_be_tested)
-            # print("????")
             for k, v in apis_to_be_tested.items():
-                # print("\n\n")
-                # print(k)
-                # print("????"+str(apis_to_be_tested[k]))
                 if not apis_to_be_tested[k]["params"]:
                     continue
                 # code = template.render(api_info=apis_to_be_tested[k], api_name=k, host_url=host_url)
@@ -474,7 +539,7 @@ class CloudvectorDAST(object):
         data = {
             '$source': code
         }
-        print(data)
+        #print(data)
         response = requests.post('https://extendsclass.com/python-tester-source', headers=headers, data=data)
         print(response.text)
         return response.json()
@@ -483,6 +548,8 @@ class CloudvectorDAST(object):
         print("\n\ncollecting events data from APIShark....")
         cv_requests = []
         events = self.ceobj.get_all_raw_events(apis_to_check)
+        print("\nNo. of events from APIShark being processed: " + str(len(events)))
+        print("\n\nprocessing events data from APIShark....")
         for event in events:
             if "http-req-header-Cv-Fuzzed-Event" in event["attributes"]["event_json"]:
                 continue
@@ -499,9 +566,7 @@ class CloudvectorDAST(object):
                 if not iflag:
                     continue
             # print(str(event["attributes"]["http_path"]))
-            request = {"url": str(event["attributes"]["event_protocol"]).lower() + "://" + \
-                              str(event["attributes"]["http_host"]) + \
-                              str(event["attributes"]["http_path"]), "method": str(event["attributes"]["http_method"])}
+            request = {"url": str(event["attributes"]["http_path"]), "method": str(event["attributes"]["http_method"]).lower()}
             header = {}
             body = {}
             req_params_found = {}
